//
//  Utilities for double precision numbers.
//
bool d_is_int ( double r );
double d_max ( double x, double y );
double d_min ( double x, double y );
double d_random ( double rlo, double rhi, int *seed );
double d_sign ( double x, double y );
void d_swap ( double *x, double *y );
double d_uniform_01 ( int *seed );
//
//  Double precision Triadiagonal.
//
double *d3_cr_fa ( int n, double a[] );
double *d3_cr_sl ( int n, double a_cr[], double b[] );
void d3_gs_sl ( int n, double a[], double b[], double x[], int it_max, int job );
double *d3_indicator ( int n );
void d3_jac_sl ( int n, double a[], double b[], double x[], int it_max, int job );
double *d3_mxv ( int n, double a[], double x[] );
double d3_np_det ( int n, double a[] );
int d3_np_fa ( int n, double a[] );
double *d3_np_fs ( int n, double a[], double b[] );
double *d3_np_ml ( int n, double a[], double x[], int job );
double *d3_np_sl ( int n, double a[], double b[], int job );
void d3_print ( int n, double a[], char *title );
void d3_print_some ( int n, double a[], int ilo, int jlo, int ihi, int jhi );
double *d3_random ( int n, int *seed );
double *d3_to_dge ( int n, double a[] );
double *d3_vxm ( int n, double a[], double x[] );
double *d3_zero ( int n );
//
//  Double precision Tridiagonal Periodic.
//
double d3p_det ( int n, double a[], double work4 );
int d3p_fa ( int n, double a[], double work2[], double work3[], double *work4 );
double *d3p_indicator ( int n );
double *d3p_ml ( int n, double a[], double x[], int job );
double *d3p_mxv ( int n, double a[], double x[] );
void d3p_print ( int n, double a[], char *title );
void d3p_print_some ( int n, double a[], int ilo, int jlo, int ihi, int jhi );
double *d3p_random ( int n, int *seed );
double *d3p_sl ( int n, double a[], double b[], int job, double work2[], 
  double work3[], double work4 );
double *d3p_to_dge ( int n, double a[] );
double *d3p_vxm ( int n, double a[], double x[] );
double *d3p_zero ( int n );
//
//  Double precision Pentagonal.
//
double *d5_indicator ( int n );
double *d5_mxv ( int n, double a[], double x[] );
double *d5_np_fs ( int n, double a[], double b[] );
void d5_print ( int n, double a[], char *title );
void d5_print_some ( int n, double a[], int ilo, int jlo, int ihi, int jhi );
double *d5_random ( int n, int *seed );
double *d5_to_dge ( int n, double a[] );
double *d5_vxm ( int n, double a[], double x[] );
double *d5_zero ( int n );
//
//  Utility to add a multiple of one real vector to another.
//
void daxpy ( int n, double sa, double x[], int incx, double y[], int incy );
//
//  Double precision Border Banded.
//
void dbb_add ( int n1, int n2, int ml, int mu, double a[], int i, int j, 
  double value );
int dbb_fa ( int n1, int n2, int ml, int mu, double a[], int pivot[] );
double dbb_get ( int n1, int n2, int ml, int mu, double a[], int i, int j );
double *dbb_indicator ( int n1, int n2, int ml, int mu );
double *dbb_mxv ( int n1, int n2, int ml, int mu, double a[], double x[] );
void dbb_print ( int n1, int n2, int ml, int mu, double a[], char *title );
void dbb_print_some ( int n1, int n2, int ml, int mu, double a[], int ilo, 
  int jlo, int ihi, int jhi );
double *dbb_random ( int n1, int n2, int ml, int mu, int *seed );
void dbb_set ( int n1, int n2, int ml, int mu, double a[], int i, int j, 
  double value );
double *dbb_sl ( int n1, int n2, int ml, int mu, double a[], int pivot[], 
  double b[] );
double *dbb_to_dge ( int n1, int n2, int ml, int mu, double a[] );
double *dbb_vxm ( int n1, int n2, int ml, int mu, double a[], double x[] );
double *dbb_zero ( int n1, int n2, int ml, int mu );
//
//  Double precision Banded Lower Triangular.
//
double dblt_det ( int n, int ml, double a[] );
double *dblt_indicator ( int n, int ml );
double *dblt_mxv ( int n, int ml, double a[], double x[] );
void dblt_print ( int n, int ml, double a[], char *title );
void dblt_print_some ( int n, int ml, double a[], int ilo, int jlo, int ihi, 
  int jhi );
double *dblt_random ( int n, int ml, int *seed );
double *dblt_sl ( int n, int ml, double a[], double b[], int job );
double *dblt_to_dge ( int n, int ml, double a[] );
double *dblt_vxm ( int n, int ml, double a[], double x[] );
double *dblt_zero ( int n, int ml );
//
//  Double precision Block Toeplitz.
//
double *dbto_indicator ( int m, int l );
double *dbto_mxv ( int m, int l, double a[], double x[] );
void dbto_print ( int m, int l, double a[], char *title );
void dbto_print_some ( int m, int l, double a[], int ilo, int jlo, int ihi, 
  int jhi );
double *dbto_random ( int m, int l, int *seed );
double *dbto_sl ( int m, int l, double a[], double b[] );
double *dbto_to_dge ( int m, int l, double a[] );
double *dbto_vxm ( int m, int l, double a[], double x[] );
double *dbto_zero ( int m, int l );
//
//  Double precision Banded Upper Triangular.
//
double dbut_det ( int n, int mu, double a[] );
double *dbut_indicator ( int n, int mu );
double *dbut_mxv ( int n, int mu, double a[], double x[] );
void dbut_print ( int n, int mu, double a[], char *title );
void dbut_print_some ( int n, int mu, double a[], int ilo, int jlo, int ihi, 
  int jhi );
double *dbut_random ( int n, int mu, int *seed );
double *dbut_sl ( int n, int mu, double a[], double b[], int job );
double *dbut_to_dge ( int n, int mu, double a[] );
double *dbut_vxm ( int n, int mu, double a[], double x[] );
//
//  Double precision Compact Banded.
//
double dcb_det ( int n, int ml, int mu, double a[] );
double *dcb_indicator ( int m, int n, int ml, int mu );
double *dcb_ml ( int n, int ml, int mu, double a[], double x[], int job );
double *dcb_mxv ( int n, int ml, int mu, double a[], double x[] );
int dcb_np_fa ( int n, int ml, int mu, double a[] );
double *dcb_np_sl ( int n, int ml, int mu, double a[], double b[], int job );
void dcb_print ( int m, int n, int ml, int mu, double a[], char *title );
void dcb_print_some ( int m, int n, int ml, int mu, double a[], int ilo, int jlo, 
  int ihi, int jhi );
double *dcb_random ( int n, int ml, int mu, int *seed );
double *dcb_to_dvec ( int m, int n, int ml, int mu, double *a );
double *dcb_to_dge ( int n, int ml, int mu, double a[] );
double *dcb_vxm ( int n, int ml, int mu, double a[], double x[] );
double *dcb_zero ( int n, int ml, int mu );
//
//  Double precision Compact Border Banded.
//
void dcbb_add ( int n1, int n2, int ml, int mu, double a[], int i, int j, 
  double value );
bool dcbb_error ( int n1, int n2, int ml, int mu );
int dcbb_fa ( int n1, int n2, int ml, int mu, double a[] );
double dcbb_get ( int n1, int n2, int ml, int mu, double a[], int i, int j );
double *dcbb_indicator ( int n1, int n2, int ml, int mu );
double *dcbb_mxv ( int n1, int n2, int ml, int mu, double a[], double x[] );
void dcbb_print ( int n1, int n2, int ml, int mu, double a[], char *title );
void dcbb_print_some ( int n1, int n2, int ml, int mu, double a[], int ilo, 
  int jlo, int ihi, int jhi );
double *dcbb_random ( int n1, int n2, int ml, int mu, int *seed );
void dcbb_set ( int n1, int n2, int ml, int mu, double a[], int i, int j, 
  double value );
double *dcbb_sl ( int n1, int n2, int ml, int mu, double a[], double b[] );
double *dcbb_to_dge ( int n1, int n2, int ml, int mu, double a[] );
double *dcbb_vxm ( int n1, int n2, int ml, int mu, double a[], double x[] );
double *dcbb_zero ( int n1, int n2, int ml, int mu );
//
//  Double precision Circulant.
//
double *dci_eval ( int n, double a[] );
double *dci_indicator ( int n );
double *dci_mxv ( int n, double a[], double x[] );
void dci_print ( int n, double a[], char *title );
void dci_print_some ( int n, double a[], int ilo, int jlo, int ihi, int jhi );
double *dci_random ( int n, int *seed );
double *dci_sl ( int n, double a[], double b[], int job );
double *dci_to_dge ( int n, double a[] );
double *dci_vxm ( int n, double a[], double x[] );
double *dci_zero ( int n );
//
//  Double precision General Banded.
//
int dgb_count_nonzeros ( int m, int n, int ml, int mu, double a[] );
double dgb_det ( int n, int ml, int mu, double a[], int pivot[] );
int dgb_fa ( int n, int ml, int mu, double a[], int pivot[] );
double *dgb_indicator ( int m, int n, int ml, int mu );
double *dgb_ml ( int n, int ml, int mu, double a[], int pivot[], double x[], 
  int job );
double *dgb_mu ( int n, int ml, int mu, double a[], int pivot[], double x[], 
  int job );
double *dgb_mxv ( int m, int n, int ml, int mu, double a[], double x[] );
void dgb_print ( int m, int n, int ml, int mu, double a[], char *title );
void dgb_print_some ( int m, int n, int ml, int mu, double a[], int ilo, 
  int jlo, int ihi, int jhi );
double *dgb_random ( int m, int n, int ml, int mu, int *seed );
double *dgb_sl ( int n, int ml, int mu, double a[], int pivot[], 
  double b[], int job );
double *dgb_to_dvec ( int m, int n, int ml, int mu, double *a );
double *dgb_to_dge ( int m, int n, int ml, int mu, double a[] );
int dgb_trf ( int m, int n, int ml, int mu, double a[], int pivot[] );
double *dgb_trs ( int n, int ml, int mu, int nrhs, char trans, double a[], 
  int pivot[], double b[] );
double *dgb_vxm ( int m, int n, int ml, int mu, double a[], double x[] );
double *dgb_zero ( int m, int n, int ml, int mu );
//
//  Double precision General Diagonal.
//
bool dgd_error ( int n, int ndiag );
double *dgd_indicator ( int n, int ndiag, int offset[] );
double *dgd_mxv ( int n, int ndiag, int offset[], double a[], double x[] );
void dgd_print ( int n, int ndiag, int offset[], double a[], char *title );
void dgd_print_some ( int n, int ndiag, int offset[], double a[], int ilo, 
  int jlo, int ihi, int jhi );
double *dgd_random ( int n, int ndiag, int offset[], int *seed );
double *dgd_to_dge ( int n, int ndiag, int offset[], double a[] );
double *dgd_vxm ( int n, int ndiag, int offset[], double a[], double x[] );
double *dgd_zero ( int n, int ndiag );
//
//  Double precision General.
//
double dge_co ( int n, double a[], int pivot[] );
double dge_det ( int n, double a[], int pivot[] );
double *dge_dilu ( int m, int n, double a[] );
int dge_fa ( int n, double a[], int pivot[] );
double *dge_fs ( int n, double a[], double b[] );
double *dge_identity ( int n );
void dge_ilu ( int m, int n, double a[], double l[], double u[] );
double *dge_indicator ( int m, int n );
double *dge_inverse ( int n, double a[], int pivot[] );
double *dge_ml ( int n, double a[], int pivot[], double x[], int job );
double *dge_mu ( int m, int n, double a[], char trans, int pivot[], double x[] );
double *dge_mxm ( int n, double a[], double b[] );
double *dge_mxv ( int m, int n, double a[], double x[] );
double dge_np_det ( int n, double a[] );
int dge_np_fa ( int n, double a[] );
double *dge_np_inverse ( int n, double a[] );
double *dge_np_ml ( int n, double a[], double x[], int job );
double *dge_np_sl ( int n, double a[], double b[], int job );
double *dge_np_trm ( int m, int n, double a[], double x[], int job );
int dge_np_trf ( int m, int n, double a[] );
double *dge_np_trs ( int n, int nrhs, char trans, double a[], double b[] );
void dge_plu ( int m, int n, double a[], double p[], double l[], double u[] );
double *dge_poly ( int n, double a[] );
void dge_print ( int m, int n, double a[], char *title );
void dge_print_some ( int m, int n, double a[], int ilo, int jlo, int ihi, 
  int jhi );
double *dge_random ( int m, int n, int *seed );
double *dge_res ( int m, int n, double a[], double x[], double b[] );
double *dge_sl ( int n, double a[], int pivot[], double b[], int job );
double *dge_sl_it ( int n, double a[], double alu[], int pivot[], double b[], 
  int job, double x[] );
double *dge_to_dgb ( int m, int n, int ml, int mu, double a[] );
double *dge_to_dvec ( int m, int n, double *a );
int dge_trf ( int m, int n, double a[], int pivot[] );
double *dge_trs ( int n, int nrhs, char trans, double a[], int pivot[], double b[] );
double *dge_vxm ( int m, int n, double a[], double x[] );
double *dge_zero ( int m, int n );
//
//  Double precision Harwell Boeing Unsymmetric Assembled.
//
double *dhbua_indicator ( int n, int nnzero, int colptr[], int rowind[] );
double *dhbua_mxv ( int n, int nnzero, int colptr[], int rowind[], 
  double a[], double x[] );
void dhbua_print ( int n, int nnzero, int colptr[], int rowind[], 
  double a[], char *title );
void dhbua_print_some ( int n, int nnzero, int colptr[], int rowind[], 
  double a[], int ilo, int jlo, int ihi, int jhi );
double *dhbua_random ( int n, int nnzero, int colptr[], int rowind[], 
  int *seed );
double *dhbua_to_dge ( int n, int nnzero, int colptr[], int rowind[], 
  double a[] );
double *dhbua_vxm ( int n, int nnzero, int colptr[], int rowind[], 
  double a[], double x[] );
double *dhbua_zero ( int n, int nnzero, int colptr[], int rowind[] );
//
//  Double precision Lower Triangular, Full Storage.
//
double dlt_det ( int n, double a[] );
double *dlt_indicator ( int m, int n );
double *dlt_inverse ( int n, double a[] );
double *dlt_mxm ( int n, double a[], double b[] );
double *dlt_mxv ( int m, int n, double a[], double x[] );
void dlt_print ( int m, int n, double a[], char *title );
void dlt_print_some ( int m, int n, double a[], int ilo, int jlo, int ihi, int jhi );
double *dlt_random ( int m, int n, int *seed );
double *dlt_sl ( int n, double a[], double b[], int job );
double *dlt_vxm ( int m, int n, double a[], double x[] );
double *dlt_zero ( int m, int n );
//
//  Utilities for double precision matrices.
//
void dmat_print ( int m, int n, double a[], char *title );
//
//  Double precision Positive Definite Symmetric Band, Lower.
//
double dpbl_det ( int n, int mu, double a[] );
double *dpbl_indicator ( int n, int mu );
void dpbl_print ( int n, int mu, double a[], char *title );
void dpbl_print_some ( int n, int mu, double a[], int ilo, int jlo, int ihi, 
  int jhi );
double *dpbl_random ( int n, int mu, int *seed );
double *dpbl_to_dge ( int n, int mu, double a[] );
double *dpbl_zero ( int n, int mu );
//
//  Double precision Positive Definite Symmetric Band, Upper.
//
double *dpbu_cg ( int n, int mu, double a[], double b[], double x[] );
double dpbu_det ( int n, int mu, double a[] );
double *dpbu_fa ( int n, int mu, double a[] );
double *dpbu_indicator ( int n, int mu );
double *dpbu_ml ( int n, int mu, double a[], double x[] );
double *dpbu_mxv ( int n, int mu, double a[], double x[] );
void dpbu_print ( int n, int mu, double a[], char *title );
void dpbu_print_some ( int n, int mu, double a[], int ilo, int jlo, int ihi, 
  int jhi );
double *dpbu_random ( int n, int mu, int *seed );
double *dpbu_sl ( int n, int mu, double a[], double b[] );
double *dpbu_sor ( int n, int mu, double a[], double b[], double eps, int itchk, 
  int itmax, double omega, double x_init[] );
double *dpbu_to_dge ( int n, int mu, double a[] );
double *dpbu_zero ( int n, int mu );
//
//  Double precision Positive Definite Symmetric.
//
double dpo_det ( int n, double a[] );
double *dpo_fa ( int n, double a[] );
double *dpo_indicator ( int n );
double *dpo_inverse ( int n, double a[] );
double *dpo_ml ( int n, double a[], double x[] );
double *dpo_mxm ( int n, double a[], double b[] );
double *dpo_mxv ( int n, double a[], double x[] );
void dpo_print ( int n, double a[], char *title );
void dpo_print_some ( int n, double a[], int ilo, int jlo, int ihi, int jhi );
double *dpo_random ( int n, int *seed );
double *dpo_sl ( int n, double a[], double b[] );
double *dpo_to_dge ( int n, double a[] );
double *dpo_zero ( int n );
//
//  Double precision Positive Definite Symmetric Packed.
//
double dpp_det ( int n, double a[] );
double *dpp_fa ( int n, double a[] );
double *dpp_indicator ( int n );
double *dpp_mxv ( int n, double a[], double x[] );
void dpp_print ( int n, double a[], char *title );
void dpp_print_some ( int n, double a[], int ilo, int jlo, int ihi, int jhi );
double *dpp_random ( int n, int *seed );
double *dpp_sl ( int n, double a[], double b[] );
double *dpp_to_dge ( int n, double a[] );
double *dpp_zero ( int n );
//
//  Utilities for double precision matrices, considered as arrays of rows.
//
void drow_swap ( int m, int n, double a[], int irow1, int irow2 );
//
//  Double precision Symmetric Diagonal
//
double *dsd_cg ( int n, int ndiag, int offset[], double a[], double b[], 
  double x[] );
double *dsd_indicator ( int n, int ndiag, int offset[] );
double *dsd_mxv ( int n, int ndiag, int offset[], double a[], double x[] );
void dsd_print ( int n, int ndiag, int offset[], double a[], char *title );
void dsd_print_some ( int n, int ndiag, int offset[], double a[], int ilo, 
  int jlo, int ihi, int jhi );
double *dsd_random ( int n, int ndiag, int offset[], int *seed );
double *dsd_to_dge ( int n, int ndiag, int offset[], double a[] );
double *dsd_zero ( int n, int ndiag );
//
//  Double precision Sherman Morrison.
//
double *dsm_ml ( int n, double a[], double u[], double v[], int pivot[], 
  double x[], int job );
double *dsm_mxv ( int m, int n, double a[], double u[], double v[], double x[] );
void dsm_print ( int m, int n, double a[], double u[], double v[], char *title );
void dsm_print_some ( int m, int n, double a[], double u[], double v[], int ilo, 
  int jlo, int ihi, int jhi );
void dsm_random ( int m, int n, double a[], double u[], double v[], int *seed );
double *dsm_sl ( int n, double a[], double u[], double v[], double b[], 
  int pivot[], int job );
double *dsm_to_dge ( int m, int n, double a[], double u[], double v[] );
double *dsm_vxm ( int m, int n, double a[], double u[], double v[], double x[] );
void dsm_zero ( int m, int n, double a[], double u[], double v[] );
//
//  Double precision Sparse storage.
//
double *dsp_indicator ( int m, int n, int nz_num, int row[], int col[] );
double *dsp_mxv ( int m, int n, int nz_num, int row[], int col[], 
  double a[], double x[] );
void dsp_print ( int m, int n, int nz_num, int row[], int col[], 
  double a[], char *title );
void dsp_print_some ( int m, int n, int nz_num, int row[], int col[], 
  double a[], int ilo, int jlo, int ihi, int jhi );
double *dsp_random ( int m, int n, int nz_num, int row[], int col[], 
  int *seed );
double *dsp_to_dge ( int m, int n, int nz_num, int row[], int col[], double a[] );
double *dsp_vxm ( int m, int n, int nz_num, int row[], int col[], 
  double a[], double x[] );
double *dsp_zero ( int m, int n, int nz_num, int row[], int col[] );
//
//  Double precision Sparse Row storage.
//
void dsr_indicator ( int n, int nz, int row[], int col[], double diag[], 
  double off[] );
double *dsr_mxv ( int n, int nz, int row[], int col[], double diag[], 
  double off[], double x[] );
void dsr_print ( int n, int nz, int row[], int col[], double diag[], 
  double off[], char *title );
void dsr_print_some ( int n, int nz, int row[], int col[], double diag[], 
  double off[], int ilo, int jlo, int ihi, int jhi );
void dsr_random ( int n, int nz, int row[], int col[], double diag[], 
  double off[], int *seed );
double *dsr_to_dge ( int n, int nz, int row[], int col[], double diag[], 
  double off[] );
double *dsr_vxm ( int n, int nz, int row[], int col[], double diag[], 
  double off[], double x[] );
void dsr_zero ( int n, int nz, int row[], int col[], double diag[], 
  double off[] );
//
//  Double precision Symmetric Skyline.
//
bool dss_error ( int diag[], int n, int na );
double *dss_indicator ( int n, int *na, int diag[]);
double *dss_mxv ( int n, int na, int diag[], double a[], double x[] );
void dss_print ( int n, int na, int diag[], double a[], char *title );
void dss_print_some ( int n, int na, int diag[], double a[], int ilo, int jlo, 
  int ihi, int jhi );
void dss_random ( int n, int *na, int diag[], double a[], int *seed );
double *dss_to_dge ( int n, int na, int diag[], double a[] );
double *dss_zero ( int n, int na, int diag[] );
//
//  Double precision Symmetric Toeplitz.
//
double *dsto_indicator ( int n );
double *dsto_inverse ( int n, double a[] );
double *dsto_mxv ( int n, double a[], double x[] );
void dsto_print ( int n, double a[], char *title );
void dsto_print_some ( int n, double a[], int ilo, int jlo, int ihi, int jhi );
double *dsto_random ( int n, int *seed );
double *dsto_sl ( int n, double a[], double b[] );
double *dsto_to_dge ( int n, double a[] );
double *dsto_yw_sl ( int n, double b[] );
double *dsto_zero ( int n );
//
//  Double precision Toeplitz.
//
double *dto_indicator ( int n );
double *dto_mxv ( int n, double a[], double x[] );
void dto_print ( int n, double a[], char *title );
void dto_print_some ( int n, double a[], int ilo, int jlo, int ihi, int jhi );
double *dto_random ( int n, int *seed );
double *dto_sl ( int n, double a[], double b[], int job );
double *dto_to_dge ( int n, double a[] );
double *dto_vxm ( int n, double a[], double x[] );
double *dto_zero ( int n );
//
//  Double precision Upper Triangular.
//
double dut_det ( int n, double a[] );
double *dut_indicator ( int m, int n );
double *dut_inverse ( int n, double a[] );
double *dut_mxm ( int n, double a[], double b[] );
double *dut_mxv ( int m, int n, double a[], double x[] );
void dut_print ( int m, int n, double a[], char *title );
void dut_print_some ( int m, int n, double a[], int ilo, int jlo, int ihi, int jhi );
double *dut_random ( int m, int n, int *seed );
double *dut_sl ( int n, double a[], double b[], int job );
double *dut_vxm ( int m, int n, double a[], double x[] );
double *dut_zero ( int m, int n );
//
//  Utilities for double precision vectors.
//
double dvec_dot ( int n, double x[], double y[] );
double *dvec_indicator ( int n );
void dvec_print ( int n, double a[], char *title );
void dvec_print_some ( int n, double a[], int max_print, char *title );
double *dvec_random ( int n, double alo, double ahi, int *seed );
double *dvec_to_dcb ( int m, int n, int ml, int mu, double *x );
double *dvec_to_dgb ( int m, int n, int ml, int mu, double *x );
double *dvec_to_dge ( int m, int n, double *x );
void dvec2_print_some ( int n, double x1[], double x2[], int max_print, 
  char *title );
//
//  Double precision Vandermonde.
//
double dvm_det ( int n, double a[] );
double *dvm_mxv ( int m, int n, double a[], double x[] );
void dvm_print ( int m, int n, double a[], char *title );
void dvm_print_some ( int m, int n, double a[], int ilo, int jlo, int ihi, int jhi );
double *dvm_random ( int m, int n, int *seed );
double *dvm_sl ( int n, double a[], double b[], int job, int *info );
double *dvm_to_dge ( int m, int n, double a[] );
double *dvm_vxm ( int m, int n, double a[], double x[] );
double *dvm_zero ( int m, int n );
//
//  Utility to get a seed for the random number generator.
//
int get_seed ( void );
//
//  Utility for one of the tests.
//
double *hilbert_inverse ( int n );
//
//  Utilities for integers.
//
int i_huge ( void );
int i_log_10 ( int i );
int i_max ( int i1, int i2 );
int i_min ( int i1, int i2 );
int i_modp ( int i, int j );
int i_random ( int ilo, int ihi, int *seed );
//
//  Utilities for integer vectors.
//
void ivec_print ( int n, int a[], char *title );

//
//  Utilities for strings.
//
int s_len_trim ( char *s );
//
//  Utility routine for external heap sort.
//
void sort_heap_external ( int n, int *indx, int *i, int *j, int isgn );
//
//  Utility to print the time.
//
void timestamp ( void );
//
//  Utility routine to compute a pseudorandom number.
//
double uniform ( int *seed );
//
//  Utilities for complex vectors.
//
void zvec_print ( int n, double a[], char *title );
void zvec_sort_a2 ( int n, double x[] );
double *zvec_unity ( int n );
